@@ -1,6 +1,6 @@
 ;;; multi-prompt.el --- Completing read of multiple strings
 
-;; Copyright (C) 1996, 1997, 2000, 2009 Free Software Foundation
+;; Copyright (C) 1996-2019 Free Software Foundation, Inc.
 
 ;; Author: Per Abrahamsen <abraham@dina.kvl.dk>
 ;; Maintainer: auctex-devel@gnu.org
@@ -11,15 +11,16 @@
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 3, or (at your option)
 ;; any later version.
-;; 
+;;
 ;; This program is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
-;; 
+;;
 ;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with AUCTeX; see the file COPYING.  If not, write to the Free
+;; Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+;; MA 02110-1301 USA.
 
 ;;; Commentary:
 
@@ -51,10 +52,7 @@ are the arguments to `completing-read'.  See that."
 		     minibuffer-local-must-match-map
 		   minibuffer-local-completion-map))
 	(new-map (make-sparse-keymap)))
-    (if (fboundp 'set-keymap-parent)
-	;; `set-keymap-parent' was introduced in Emacs 19.32.
-	(set-keymap-parent new-map old-map)
-      (setq new-map (copy-keymap old-map)))
+    (set-keymap-parent new-map old-map)
     (define-key new-map separator (if require-match
 				      'multi-prompt-next-must-match
 				    'multi-prompt-next))
@@ -62,7 +60,6 @@ are the arguments to `completing-read'.  See that."
     (let* ((minibuffer-local-completion-map new-map)
 	   (minibuffer-local-must-match-map new-map)
 	   (multi-prompt-found nil)
-	   (done nil)
 	   (filter (cond (unique
 			  (lambda (x)
 			    (and (not (member (car x) multi-prompt-found))
@@ -94,9 +91,9 @@ are the arguments to `completing-read'.  See that."
 				      initial nil)
 				(setq multi-prompt-found
 				      (cons extra multi-prompt-found)))))))))
-      (if answer 
-	  (nreverse (cons answer multi-prompt-found))
-	multi-prompt-found))))
+      (if (string= answer "")
+	  multi-prompt-found
+	(nreverse (cons answer multi-prompt-found))))))
 
 (defun multi-prompt-delete ()
   (interactive)
@@ -119,8 +116,8 @@ are the arguments to `completing-read'.  See that."
   (interactive)
   (when  (call-interactively 'minibuffer-complete)
     (let ((content (buffer-substring-no-properties (point-min) (point-max))))
-      (when (or (not require-match)
-		(assoc content table))
+      (when (or ;; (not require-match)
+                (assoc content minibuffer-completion-table))
 	(throw 'multi-prompt-next content)))))
 
 
@@ -200,7 +197,6 @@ other arguments: PREDICATE, REQUIRE-MATCH, INITIAL-INPUT, HIST,
 DEF, and INHERIT-INPUT-METHOD.
 
 The return value is the string as entered in the minibuffer."
-  (require 'crm)
   (let* ((minibuffer-completion-table #'multi-prompt-key-value-collection-fn)
 	 (minibuffer-completion-predicate predicate)
 	 (minibuffer-completion-confirm
