@@ -1,6 +1,6 @@
 ;;; csquotes.el --- AUCTeX style for `csquotes.sty'
 
-;; Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2014, 2018 Free Software Foundation, Inc.
 
 ;; Author: Ralf Angeli <angeli@caeruleus.net>
 ;; Maintainer: auctex-devel@gnu.org
@@ -28,9 +28,13 @@
 
 ;; This file adds support for `csquotes.sty', version 3.7.
 
-
 ;;; Code:
 
+;; Silence the compiler:
+(declare-function font-latex-add-keywords
+		  "font-latex"
+		  (keywords class))
+
 ;; FIXME: It would be nice to be able to dump this function in favor
 ;; of a generalized handling of additional arguments for environments
 ;; specified via `LaTeX-add-environments'.  `TeX-parse-arguments' and
@@ -57,7 +61,7 @@ the insertion of optional arguments."
 		optional t)
 	(setq prompt elt
 	      optional nil))
-      (setq user-input (read-string (TeX-argument-prompt optional prompt nil)))
+      (setq user-input (TeX-read-string (TeX-argument-prompt optional prompt nil)))
       (unless (and optional (zerop (length user-input)))
 	(setq env-extra (concat env-extra
 				(if optional LaTeX-optop TeX-grop)
