@@ -1,6 +1,6 @@
 ;;; newfloat.el --- AUCTeX style for `newfloat.sty' (v1.1-109)
 
-;; Copyright (C) 2015 Free Software Foundation, Inc.
+;; Copyright (C) 2015, 2018 Free Software Foundation, Inc.
 
 ;; Author: Arash Esbati <arash@gnu.org>
 ;; Maintainer: auctex-devel@gnu.org
@@ -54,9 +54,14 @@
 
 ;;; Code:
 
-;; Needed for auto-parsing.
+;; Needed for auto-parsing:
 (require 'tex)
 
+;; Silence the compiler:
+(declare-function font-latex-add-keywords
+		  "font-latex"
+		  (keywords class))
+
 (defvar LaTeX-newfloat-key-val-options
   '(("fileext")
     ("listname")
@@ -119,7 +124,7 @@ If `caption.el' is loaded, add the new floating environment to
 	     (LaTeX-add-environments `(,flt ["Float Position"]))
 	     (add-to-list (make-local-variable 'LaTeX-indent-environment-list)
 			  `(,flt current-indentation) t)
-	     (add-to-list 'LaTeX-label-alist `(,flt . "lst:") t)
+	     (add-to-list 'LaTeX-label-alist `(,flt . LaTeX-listing-label) t)
 	     (when (fboundp 'reftex-add-label-environments)
 	       (reftex-add-label-environments
 		`((,flt ?l "lst:" "~\\ref{%s}" caption nil nil)))))
