@@ -1,6 +1,6 @@
 ;;; pstricks.el --- AUCTeX style for the `pstricks' package.
 
-;; Copyright (C) 2007, 2009, 2013-2015 Free Software Foundation, Inc.
+;; Copyright (C) 2007, 2009, 2013-2015, 2018  Free Software Foundation, Inc.
 
 ;; Author: Holger Sparr <holger.sparr@gmx.net>
 ;; Maintainer: auctex-devel@gnu.org
@@ -48,7 +48,7 @@
 ;;
 ;; 14/06/2007 rewrite of pstricks.el based on Jean-Philippe Georget's
 ;;            pstricks.el version found on <URI:
-;;            http://www.emacswiki.org/cgi-bin/wiki/pstricks.el>
+;;            https://www.emacswiki.org/emacs/pstricks.el>
 
 ;;; TODO:
 ;;
@@ -60,6 +60,9 @@
 
 ;;; Code:
 
+(eval-when-compile
+  (require 'cl-lib))
+
 ;;; General Functions
 
 (defun TeX-arg-compl-list (list &optional prompt hist)
@@ -228,7 +231,7 @@ package PNAME"
 
 (defun LaTeX-pst-point-in-parens (_optional)
   "Enclose point in parentheses."
-  (LaTeX-pst-enclose-obj 'LaTeX-pst-point ?( ?)))
+  (LaTeX-pst-enclose-obj 'LaTeX-pst-point ?\( ?\)))
 
 ;;; Angles
 (defvar LaTeX-pst-angle-list (list "0")
@@ -681,7 +684,7 @@ package PNAME"
        (cond ((string= type "object")
               (setq TeX-auto-symbol
                     (cons (list (nth 1 list)
-                                (caddr (assoc (nth 2 list)
+                                (cl-caddr (assoc (nth 2 list)
                                               (TeX-symbol-list))))
                           TeX-auto-symbol)))
              ((string= type "fontdot")
@@ -716,7 +719,7 @@ comma separated list. Point has to be within the sexp to modify."
         (progn
           (re-search-backward "\\\\\\([a-zA-Z]\\)")
           (forward-word 1)
-          (insert-pair nil ?[ ?]))
+          (insert-pair nil ?\[ ?\]))
       (up-list 1)
       (backward-char 1)
       (save-excursion
