@@ -1,6 +1,6 @@
 ;;; siunitx.el --- AUCTeX style for `siunitx.sty' version 2.5s.
 
-;; Copyright (C) 2012-2014 Free Software Foundation, Inc.
+;; Copyright (C) 2012-2015, 2018 Free Software Foundation, Inc.
 
 ;; Maintainer: auctex-devel@gnu.org
 ;; Author: Mosè Giordano <giordano.mose@libero.it>
@@ -29,6 +29,13 @@
 
 ;;; Code:
 
+(require 'tex) ;Indispensable when compiling the call to `TeX-auto-add-type'.
+
+;; Silence the compiler:
+(declare-function font-latex-add-keywords
+		  "font-latex"
+		  (keywords class))
+
 (TeX-auto-add-type "siunitx-unit" "LaTeX")
 
 ;; Self Parsing -- see (info "(auctex)Hacking the Parser").  `\\(?:\\[.*\\]\\)?'
@@ -54,9 +61,9 @@
 	    (add-to-list 'LaTeX-siunitx-unit-list (list symbol)))
 	  LaTeX-auto-siunitx-unit))
 
-;; FIXME: This does not seem to work unless one does a manual reparse.
-(add-hook 'TeX-auto-prepare-hook 'LaTeX-siunitx-prepare)
-(add-hook 'TeX-auto-cleanup-hook 'LaTeX-siunitx-cleanup)
+(add-hook 'TeX-auto-prepare-hook #'LaTeX-siunitx-prepare t)
+(add-hook 'TeX-auto-cleanup-hook #'LaTeX-siunitx-cleanup t)
+(add-hook 'TeX-update-style-hook #'TeX-auto-parse t)
 
 (defvar LaTeX-siunitx-unit-history nil
   "History of units in siunitx.")
@@ -588,11 +595,17 @@ string."
       "ab"
       "zb"
       "yb"))
+
+   ;; `siunitx.sty' adds new column specification letters
+   (set (make-local-variable 'LaTeX-array-column-letters)
+	(concat LaTeX-array-column-letters "s" "S"))
+
    (TeX-run-style-hooks "l3keys2e"
 			"array"
 			"amstext"
 			"xparse"
 			"expl3")
+
    ;; Fontification
    (when (and (featurep 'font-latex)
 	      (eq TeX-install-font-lock 'font-latex-setup))
