@@ -1,6 +1,6 @@
 ;;; tex-fold.el --- Fold TeX macros.
 
-;; Copyright (C) 2004-2008, 2011-2012, 2014, 2017
+;; Copyright (C) 2004-2008, 2011-2012, 2014, 2017, 2018
 ;;   Free Software Foundation, Inc.
 
 ;; Author: Ralf Angeli <angeli@caeruleus.net>
@@ -44,10 +44,9 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
+(eval-when-compile
+  (require 'cl-lib))
 
-(when (featurep 'xemacs)
-  (require 'overlay))
 (require 'tex)
 (autoload 'LaTeX-forward-paragraph "latex")
 (autoload 'LaTeX-backward-paragraph "latex")
@@ -336,8 +335,8 @@ for macros and 'math for math macros."
 			  ((eq type 'math) TeX-fold-math-spec-list-internal)
 			  (t TeX-fold-macro-spec-list-internal)))
 	(dolist (i (cadr item))
-          (pushnew (list i (car item)) fold-list :test #'equal)
-	  (pushnew i item-list :test #'equal)))
+          (cl-pushnew (list i (car item)) fold-list :test #'equal)
+	  (cl-pushnew i item-list :test #'equal)))
       (when item-list
 	(setq regexp (cond ((and (eq type 'env)
 				 (eq major-mode 'context-mode))
@@ -570,22 +569,21 @@ TYPE can be either 'env for environments, 'macro for macros or
   "Return t if an overfull line will result after adding an overlay.
 The overlay extends from OV-START to OV-END and will display the
 string DISPLAY-STRING."
-  (and (not (featurep 'xemacs)) ; Linebreaks in glyphs don't
-				; work in XEmacs anyway.
-       (save-excursion
-	 (goto-char ov-end)
-	 (search-backward "\n" ov-start t))
-       (not (string-match "\n" display-string))
-       (> (+ (- ov-start
-		(save-excursion
-		  (goto-char ov-start)
-		  (line-beginning-position)))
-	     (length display-string)
-	     (- (save-excursion
-		  (goto-char ov-end)
-		  (line-end-position))
-		ov-end))
-	  (current-fill-column))))
+  (and
+   (save-excursion
+     (goto-char ov-end)
+     (search-backward "\n" ov-start t))
+   (not (string-match "\n" display-string))
+   (> (+ (- ov-start
+	    (save-excursion
+	      (goto-char ov-start)
+	      (line-beginning-position)))
+	 (length display-string)
+	 (- (save-excursion
+	      (goto-char ov-end)
+	      (line-end-position))
+	    ov-end))
+      (current-fill-column))))
 
 (defun TeX-fold-macro-nth-arg (n macro-start &optional macro-end delims)
   "Return a property list of the argument number N of a macro.
@@ -597,11 +595,8 @@ opening and closing syntax as defined in
 `TeX-search-syntax-table'.
 
 The first item in the returned list is the string specified in
-the argument, the second item may be a face if the argument
-string was fontified.  In Emacs the string holds text properties
-as well, so the second item is always nil.  In XEmacs the string
-does not enclose any faces, so these are given in the second item
-of the resulting list."
+the argument, with text properties.  The second item is for
+backward compatibility and always nil."
   (save-excursion
     (let* ((macro-end (or macro-end
 			  (save-excursion (goto-char macro-start)
@@ -635,12 +630,7 @@ of the resulting list."
 		  (setq n (1- n)))
 		t)
 	    (error nil))
-	  (list (TeX-fold-buffer-substring content-start content-end)
-		(when (and (featurep 'xemacs)
-			   (extent-at content-start))
-		  ;; A glyph in XEmacs does not seem to be able to hold more
-		  ;; than one face, so we just use the first one we get.
-		  (car (extent-property (extent-at content-start) 'face))))
+	  (list (TeX-fold-buffer-substring content-start content-end))
 	nil))))
 
 (defun TeX-fold-buffer-substring (start end)
@@ -710,7 +700,7 @@ breaks will be replaced by spaces."
     (dolist (ov (overlays-at (point)))
       (when (and (eq (overlay-get ov 'category) 'TeX-fold)
 		 (numberp (overlay-get ov 'TeX-fold-display-string-spec)))
-	(pushnew ov overlays)))
+	(cl-pushnew ov overlays)))
     (when overlays
       ;; Sort list according to descending starts.
       (setq overlays (sort (copy-sequence overlays)
@@ -807,7 +797,7 @@ That means, put respective properties onto overlay OV."
 		     (TeX-fold-expand-spec spec ov-start ov-end))
 		    ((functionp spec)
 		     (let (arg arg-list
-			   (n 1))
+			       (n 1))
 		       (while (setq arg (TeX-fold-macro-nth-arg
 					 n ov-start ov-end))
                          (unless (member (car arg) arg-list)
@@ -821,28 +811,20 @@ That means, put respective properties onto overlay OV."
 			   "[Error: No content found]"))))
 	 (display-string (if (listp computed) (car computed) computed))
 	 (face (when (listp computed) (cadr computed))))
-    ;; Cater for zero-length display strings.
-    (when (string= display-string "") (setq display-string TeX-fold-ellipsis))
-    ;; Add a linebreak to the display string and adjust the overlay end
-    ;; in case of an overfull line.
-    (when (TeX-fold-overfull-p ov-start ov-end display-string)
-      (setq display-string (concat display-string "\n"))
-      (move-overlay ov ov-start (save-excursion
-				  (goto-char ov-end)
-				  (skip-chars-forward " \t")
-				  (point))))
-    (overlay-put ov 'mouse-face 'highlight)
-    (overlay-put ov 'display display-string)
-    (if (featurep 'xemacs)
-	(let ((glyph (make-glyph (if (listp display-string)
-				     (car display-string)
-				   display-string))))
-	  (overlay-put ov 'invisible t)
-	  (when font-lock-mode
-	    (if face
-		(set-glyph-property glyph 'face face)
-	      (set-glyph-property glyph 'face TeX-fold-folded-face)))
-	  (set-extent-property ov 'end-glyph glyph))
+    ;; Do nothing if the overlay is empty.
+    (when (and ov-start ov-end)
+      ;; Cater for zero-length display strings.
+      (when (string= display-string "") (setq display-string TeX-fold-ellipsis))
+      ;; Add a linebreak to the display string and adjust the overlay end
+      ;; in case of an overfull line.
+      (when (TeX-fold-overfull-p ov-start ov-end display-string)
+	(setq display-string (concat display-string "\n"))
+	(move-overlay ov ov-start (save-excursion
+				    (goto-char ov-end)
+				    (skip-chars-forward " \t")
+				    (point))))
+      (overlay-put ov 'mouse-face 'highlight)
+      (overlay-put ov 'display display-string)
       (when font-lock-mode
 	(overlay-put ov 'face TeX-fold-folded-face))
       (unless (zerop TeX-fold-help-echo-max-length)
@@ -853,14 +835,10 @@ That means, put respective properties onto overlay OV."
   "Show a single LaTeX macro or environment.
 Remove the respective properties from the overlay OV."
   (overlay-put ov 'mouse-face nil)
-  (if (featurep 'xemacs)
-      (progn
-	(set-extent-property ov 'end-glyph nil)
-	(overlay-put ov 'invisible nil))
-    (overlay-put ov 'display nil)
-    (overlay-put ov 'help-echo nil)
-    (when font-lock-mode
-      (overlay-put ov 'face TeX-fold-unfolded-face))))
+  (overlay-put ov 'display nil)
+  (overlay-put ov 'help-echo nil)
+  (when font-lock-mode
+    (overlay-put ov 'face TeX-fold-unfolded-face)))
 
 ;; Copy and adaption of `reveal-post-command' from reveal.el in GNU
 ;; Emacs on 2004-07-04.
@@ -904,10 +882,7 @@ Remove the respective properties from the overlay OV."
 	      ;; Close old overlays.
 	      (dolist (ol old-ols)
 		(when (and (eq (current-buffer) (overlay-buffer ol))
-			   (not (rassq ol TeX-fold-open-spots))
-			   (or (not (featurep 'xemacs))
-			       (and (featurep 'xemacs)
-				    (not (extent-detached-p ol)))))
+			   (not (rassq ol TeX-fold-open-spots)))
 		  (if (and (>= (point) (overlay-start ol))
 			   (<= (point) (overlay-end ol)))
 		      ;; Still near the overlay: keep it open.
@@ -935,12 +910,7 @@ the other elements.  The ordering among elements is maintained."
 
 ;;; The mode
 
-;; This autoload cookie had to be changed because of XEmacs.  This is
-;; very dissatisfactory, because we now don't have the full doc string
-;; available to tell people what to expect when using this mode before
-;; loading it.
-
-;;;###autoload (autoload 'TeX-fold-mode "tex-fold" "Minor mode for hiding and revealing macros and environments." t)
+;;;###autoload
 (define-minor-mode TeX-fold-mode
   "Minor mode for hiding and revealing macros and environments.
 
