@@ -1,10 +1,12 @@
-;;; tex-jp.el --- Support for Japanese TeX.
+;;; tex-jp.el --- Support for Japanese TeX.  -*- coding: iso-2022-jp-unix; -*-
 
-;; Copyright (C) 1999, 2001 Hidenobu Nabetani <nabe@debian.or.jp>
-;; Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation
+;; Copyright (C) 1999, 2001-2008, 2012-2013, 2016-2018
+;;   Free Software Foundation, Inc.
 
-;; Author:     KOBAYASHI Shinji <koba@flab.fujitsu.co.jp>
+;; Author:     KOBAYASHI Shinji <koba@flab.fujitsu.co.jp>,
+;;             Hidenobu Nabetani <nabe@debian.or.jp>
 ;; Maintainer: Masayuki Ataka <masayuki.ataka@gmail.com>
+;;             Ikumi Keita <ikumikeita@jcom.home.ne.jp>
 ;; Keywords: tex
 
 ;; This file is part of AUCTeX.
@@ -33,12 +35,14 @@
 ;;; Code:
 
 (require 'latex)
+(require 'tex-buf)
 
 ;;; Customization
 
 (defgroup AUCTeX-jp nil
   "Japanese support in AUCTeX."
-  :group 'AUCTeX)
+  :group 'AUCTeX
+  :link '(custom-manual "(auctex)Japanese"))
 
 (defcustom japanese-TeX-engine-default 'ptex
   "Default TeX engine for Japanese TeX."
@@ -47,84 +51,53 @@
 		 (const :tag "jTeX" jtex)
 		 (const :tag "upTeX" uptex)))
 
+(defcustom japanese-TeX-use-kanji-opt-flag t
+  "Add kanji option to Japanese pTeX family if non-nil.
+If `TeX-japanese-process-input-coding-system' or
+`TeX-japanese-process-output-coding-system' are non-nil, the process coding
+systems are determined by their values regardless of the kanji option."
+  :group 'AUCTeX-jp
+  :type 'boolean)
+
 (setq TeX-engine-alist-builtin
       (append TeX-engine-alist-builtin
              '((ptex "pTeX" "ptex %(kanjiopt)" "platex %(kanjiopt)" "eptex")
                (jtex "jTeX" "jtex" "jlatex" nil)
                (uptex "upTeX" "euptex" "uplatex" "euptex"))))
 
-;; $B=gD4$K9T$1$PITMW$K$J$k!#(B
-(defcustom japanese-TeX-command-list
-  ;; Changed to double quotes for Windows afflicted people.  I don't
-  ;; use the %(latex) and %(tex) shorthands here because I have not
-  ;; clue whether Omega-related versions exist.  --dak
-  '(("jTeX" "%(PDF)jtex %`%S%(PDFout)%(mode)%' %t"
-     TeX-run-TeX nil (plain-tex-mode) :help "Run NTT jTeX")
-    ("jLaTeX" "%(PDF)jlatex %`%S%(PDFout)%(mode)%' %t"
-     TeX-run-TeX nil (latex-mode) :help "Run NTT jLaTeX")
-    ("pTeX" "%(PDF)ptex %(kanjiopt)%`%S%(PDFout)%(mode)%' %t"
-     TeX-run-TeX nil (plain-tex-mode) :help "Run ASCII pTeX")
-    ("pLaTeX" "%(PDF)platex %(kanjiopt)%`%S%(PDFout)%(mode)%' %t"
-     TeX-run-TeX nil (latex-mode) :help "Run ASCII pLaTeX")
-    ("Mendex" "mendex %(mendexkopt)%s" TeX-run-command nil t :help "Create index file with mendex")
-    ("jBibTeX" "jbibtex %s" TeX-run-BibTeX nil t :help "Run jBibTeX")
-    ("pBibTeX" "pbibtex %(kanjiopt)%s" TeX-run-BibTeX nil t :help "Run pBibTeX"))
-  "Additional list of commands, especially for Japanese.
-For detail, see `TeX-command-list', to which this list is appended."
-  :group 'AUCTeX-jp
-  :type '(repeat (group :value ("" "" TeX-run-command nil t)
-			(string :tag "Name")
-			(string :tag "Command")
-			(choice :tag "How"
-				:value TeX-run-command
-				(function-item TeX-run-command)
-				(function-item TeX-run-format)
-				(function-item TeX-run-TeX)
-				(function-item TeX-run-interactive)
-				(function-item TeX-run-BibTeX)
-				(function-item TeX-run-compile)
-				(function-item TeX-run-shell)
-				(function-item TeX-run-discard)
-				(function-item TeX-run-background)
-				(function-item TeX-run-silent)
-				(function-item TeX-run-discard-foreground)
-				(function-item TeX-run-function)
-				(function-item TeX-run-discard-or-function)
-				(function :tag "Other"))
-			(boolean :tag "Prompt")
-			(choice :tag "Modes"
-				(const :tag "All" t)
-				(set (const :tag "Plain TeX" plain-tex-mode)
-				     (const :tag "LaTeX" latex-mode)
-				     (const :tag "DocTeX" doctex-mode)
-				     (const :tag "ConTeXt" context-mode)
-				     (const :tag "Texinfo" texinfo-mode)
-				     (const :tag "AmSTeX" ams-tex-mode)))
-			(repeat :tag "Menu elements" :inline t sexp))))
-
-;; $B=gD4$K9T$1$PITMW$K$J$k!#(B
+;; customize option $B$N=i4|CM$d(B saved value $B$=$N$b$N$r2~JQ$7$J$$$h$&$K(B
+;; $B$9$k$?$a!"(Bsetcar $B$N;HMQ$OHr$1$k!#(B
 (setq TeX-command-list
-      (append japanese-TeX-command-list
-	      '(("-" "" ignore nil t)) ;; separator for command menu
-	      TeX-command-list))
+      ;; `TeX-command-list' $B$HF1$89=B$$N?7$7$$(B list $B$r:n$k!#(B
+      ;; $B3FMWAG$N(B list $B$r(B l $B$H$7$F!"(Bl $B$=$N$b$N$r;H$C$?$j!"(Bl $B$r(B
+      ;; $B<c43=$@5$7$?(B list $B$r:n$C$?$j$7$F(B `mapcar' $B$G=8$a$k!#(B
+      (mapcar
+       (lambda (l)
+	 (cond
+	  ;; l $B$NBh(B1$BMWAG$,(B "BibTeX" $B$d(B "Index" $B$@$C$?$i!"(Bl $B$NBh(B2$BMWAG(B
+	  ;; $B$@$1$rF~$lBX$($?JL$N(B list $B$r:n$k!#(B
+	  ((equal (car l) "BibTeX")
+	   (append (list (car l) "%(bibtex) %s") (cddr l)))
+	  ((equal (car l) "Index")
+	   (append (list (car l) "%(makeindex) %s") (cddr l)))
+	  ;; $B$=$l0J30$N>l9g$O(B l $B$=$N$b$N$r;H$&!#(B
+	  (t
+	   l)))
+       TeX-command-list))
 
-;; $B;CDj=hCV!#(Btex.el $B$K<h$j9~$s$G$b$i$($k$H$h$$!#(B
-(setcar (cdr (assoc "BibTeX" TeX-command-list)) "%(bibtex) %s")
-(setcar (cdr (assoc "Index" TeX-command-list)) "%(makeindex) %s")
+;; Define before first use.
+(defvar japanese-TeX-mode nil
+  "Non-nil means the current buffer handles Japanese TeX/LaTeX.")
+(make-variable-buffer-local 'japanese-TeX-mode)
+(put 'japanese-TeX-mode 'permanent-local t)
 
-;; $B;CDj=hCV!#(Btex.el $B$K<h$j9~$s$G$b$i$($k$H$h$$!#(B
-(setq TeX-expand-list
+(setq TeX-expand-list-builtin
       (append
-       TeX-expand-list
+       TeX-expand-list-builtin
        '(
         ;; -kanji $B%*%W%7%g%s$NJ8;zNs$r:n$k!#(B
         ("%(kanjiopt)" (lambda ()
-                         (if (and
-                              ;; non-mule $B$J(B emacsen $B$O$=$b$=$bF|K\8l(B
-                              ;; $BJ8=q$r(B typeset $B$9$k$3$H$O9M$($J$/$F$b(B
-                              ;; $B$$$$$@$m$&!"$H$O;W$&$1$I0l1~!D!#(B
-                              (featurep 'mule)
-                              japanese-TeX-use-kanji-opt-flag)
+                         (if japanese-TeX-use-kanji-opt-flag
                              (let ((str (japanese-TeX-get-encoding-string)))
                                (if str (format " -kanji=%s " str) ""))
                            "")))
@@ -140,19 +113,44 @@ For detail, see `TeX-command-list', to which this list is appended."
                         ((eq TeX-engine 'jtex) "jbibtex")
                         ((eq TeX-engine 'uptex) "upbibtex")
                         (t "bibtex"))))
-        ;; mendex $B$H(B makeindex $B$NE,@Z$JJ}$rA*Br$9$k!#(B
+        ;; mendex, upmendex, makeindex $B$N$&$AE,@Z$J$b$N$rA*Br$9$k!#(B
         ("%(makeindex)" (lambda ()
-                          (if (memq TeX-engine '(ptex uptex))
-                              "mendex %(mendexkopt)" "makeindex")))
+                          (cond
+			   ;; upmendex $B$O(B XeLaTeX $B$d(B LuaLaTeX $B$G$b(B
+			   ;; $B;H$($k!#(Bsee
+			   ;; http://www.t-lab.opal.ne.jp/tex/README_upmendex.md
+			   ;; FIXME: XeLaTeX $B$d(B LuaLaTeX $B$@$1$r;H$&(B
+			   ;; $BMxMQ<T$N>l9g$O!"(Btex-jp $B$O(B load $B$5$l$J(B
+			   ;; $B$$$N$G$3$N@_Dj$O0UL#$,$J$$!#(B
+                           ((and (memq TeX-engine '(uptex xetex luatex))
+                                 (executable-find "upmendex"))
+                            "upmendex %(dic)")
+                           ((memq TeX-engine '(ptex uptex))
+                            "mendex %(mendexkopt) %(dic)")
+                           (t "makeindex"))))
         ;; mendex $BMQF|K\8l%3!<%I%*%W%7%g%s!#(B
         ("%(mendexkopt)" (lambda ()
-                           (if (and (featurep 'mule)
-                                    japanese-TeX-use-kanji-opt-flag)
+                           (if japanese-TeX-use-kanji-opt-flag
                                (let ((str (japanese-TeX-get-encoding-string)))
                                  ;; $B#1J8;zL\$rBgJ8;z$K!#(B
                                  (if str (format " -%c " (upcase (aref str 0)))
                                    ""))
                              "")))
+	;; (up)mendex $BMQ<-=q;XDj%*%W%7%g%s!#(B
+	("%(dic)" (lambda ()
+		    ;; master $B$HF1L>$G3HD%;R$,(B .dic $B$N%U%!%$%k$,$"$l$P(B
+		    ;; $B$=$l$r<-=qL>$H$7$F(B -d $B%*%W%7%g%s$KM?$($k!#(B
+		    ;; C-c C-r $BEy$N>l9g(B _region_.dic $B$K$9$Y$-$G$O(B
+		    ;; $B$J$$$N$G!"(B`TeX-master-file' $B$rM[$K8F$V!#(B
+		    (let ((dicname (TeX-master-file "dic" t)))
+		      (if (file-exists-p
+			   (expand-file-name dicname (TeX-master-directory)))
+			  (let ((result (format "-d %s" dicname)))
+			    ;; Advance past the file name in order to
+			    ;; prevent expanding any substring of it.
+			    (setq pos (+ pos (length result)))
+			    result)
+			""))))
         ;; pxdvi $B$H(B %(o?)xdvi $B$NE,@Z$JJ}$rA*Br$9$k!#(B
         ("%(xdvi)" (lambda ()
                      ;; pxdvi $B$O(B ptex, jtex $B6&MQ$J$N$G!"(B
@@ -162,89 +160,57 @@ For detail, see `TeX-command-list', to which this list is appended."
 
 ;;; Viewing (new implementation)
 
-(unless (get 'TeX-view-predicate-list 'saved-value)
-  (setq TeX-view-predicate-list
+(setq TeX-view-predicate-list-builtin
+      (append
        '((paper-a4
-          (TeX-match-style
-           "\\`\\(a4j\\|a4paper\\|a4dutch\\|a4wide\\|sem-a4\\)\\'"))
+	  (let ((regex "\\`\\(?:a4j\\|a4paper\\|a4dutch\\|a4wide\\|sem-a4\\)\\'"))
+	    (or (TeX-match-style regex)
+		(and (fboundp 'LaTeX-match-class-option)
+		     (LaTeX-match-class-option regex)))))
          (paper-a5
-          (TeX-match-style
-           "\\`\\(a5j\\|a5paper\\|a5comb\\)\\'"))
+	  (let ((regex "\\`\\(?:a5j\\|a5paper\\|a5comb\\)\\'"))
+	    (or (TeX-match-style regex)
+		(and (fboundp 'LaTeX-match-class-option)
+		     (LaTeX-match-class-option regex)))))
          ;; jarticle $B$J$I$@$H(B b4paper, b5paper $B$O(B JIS B $B7ONs!#(B
          ;; j-article $B$J$I$NJ}$K$O(B a4j, b5j $B$H$$$C$?%*%W%7%g%s$O$J$$!#(B
          (paper-b5    ; ISO B5
-          (and (TeX-match-style "\\`b5paper\\'")
-               (not (memq TeX-engine '(ptex uptex)))))
+          (and (fboundp 'LaTeX-match-class-option)
+	       (LaTeX-match-class-option "\\`b5paper\\'")
+               (not (TeX-match-style "\\`u?[jt]s?\\(?:article\\|report\\|book\\)\\'"))))
          (paper-b5jis ; JIS B5
-          (or (TeX-match-style "\\`b5j\\'")
-              (and (TeX-match-style "\\`b5paper\\'")
-                   (memq TeX-engine '(ptex uptex)))))
+	  (and (fboundp 'LaTeX-match-class-option)
+	       (or (LaTeX-match-class-option "\\`b5j\\'")
+		   (and (LaTeX-match-class-option "\\`b5paper\\'")
+			(TeX-match-style "\\`u?[jt]s?\\(?:article\\|report\\|book\\)\\'")))))
          ;; article $B$J$I$K$O(B b4paper $B$H$$$&%*%W%7%g%s$O$J$$!#(B
          ;; b4paper $B$H$$$&%*%W%7%g%s$,$"$C$?$i(B JIS B4 $B$H8+$J$9!#(B
          (paper-b4jis
-          (TeX-match-style "\\`\\(b4j\\|b4paper\\)\\'")))))
-;; jsarticle $B$@$HB>$K$b$b$C$HH=7?$N%*%W%7%g%s$,$"$k$,!"(B
-;; $BA4ItLLE]8+$F$k$H%-%j$,$J$$$N$G!"$3$l$/$i$$$G$$$$$@$m$&!#(B
-;; jsarticle.el $B$d(B jsbook.el $B$GDI2CJ,$N=hM}$r;E9~$a$P$$$$$N$+$bCN$l$J$$!#(B
+	  (and (fboundp 'LaTeX-match-class-option)
+	       (LaTeX-match-class-option "\\`\\(?:b4j\\|b4paper\\)\\'"))))
+       ;; jsclasses $B$@$HB>$K$b$b$C$HH=7?$N%*%W%7%g%s$,$"$k$,!"A4ItLLE](B
+       ;; $B8+$F$k$H%-%j$,$J$$$N$G!"$3$l$/$i$$$G$$$$$@$m$&!#(B
+       ;; jsarticle.el $B$d(B jsbook.el $B$GDI2CJ,$N=hM}$r;E9~$a$P$$$$$N$+$bCN$l$J$$!#(B
+       TeX-view-predicate-list-builtin))
 
-;; $B;CDj=hCV!#(Btex.el $B$K<h$j9~$s$G$b$i$($k$H$h$$!#(B
-(unless (get 'TeX-view-program-list 'saved-value)
-  (setq TeX-view-program-list
-       (cond
-        ;; http://oku.edu.mie-u.ac.jp/~okumura/texwiki/?AUCTeX
-        ;; $B$r;29M$K$7$F$_$?!#(B
-        ((eq system-type 'windows-nt)
-         '(("Dviout" ("dviout -1 "
-                      ((paper-a4 paper-portrait) " -y=A4 ")
-                      ((paper-a4 paper-landscape) " -y=A4L ")
-                      ((paper-a5 paper-portrait) " -y=A5 ")
-                      ((paper-a5 paper-landscape) " -y=A5L ")
-                      ((paper-b5 paper-portrait) " -y=E5 ")
-                      ((paper-b5 paper-landscape) " -y=E5L ")
-                      ((paper-b4jis paper-portrait) " -y=B4 ")
-                      ((paper-b4jis paper-landscape) " -y=B4L ")
-                      ((paper-b5jis paper-portrait) " -y=B5 ")
-                      ((paper-b5jis paper-landscape) " -y=B5L ")
-                      (paper-legal " -y=Legal ")
-                      (paper-letter " -y=Letter ")
-                      (paper-executive " -y=Exective ")
-                      "%o" (mode-io-correlate " \"# %n '%b'\"")))
-           ("TeXworks" "TeXworks %o")
-           ("SumatraPDF" "SumatraPDF -reuse-instance %o"
-            (mode-io-correlate " -forward-search \"%b\" %n"))
-           ("MuPDF" "mupdf %o")))
-        ;; $B$3$l$G$$$$$N$+$I$&$+$OIT0B!#(B
-        ((eq system-type 'darwin)
-         '(("Preview" "open -a Preview.app %o")
-           ("TeXShop" "open -a TeXShop.app %o")
-           ("TeXworks" "open -a TeXworks.app %o")
-           ("Skim" "open -a Skim.app %o")
-           ("displayline" "displayline %n %o %b")
-           ("PictPrinter" "open -a PictPrinter.app %d")
-           ("Mxdvi" "open -a Mxdvi.app %d")
-           ("open" "open %o")))
-        (t
-         (setcar (cadr (assoc "xdvi" TeX-view-program-list-builtin))
-                 "%(xdvi) -unique")
-         '(("TeXworks" "texworks %o")
-           ("zathura" "zathura %o")
-           ("MuPDF" "mupdf %o"))))))
+(unless (memq system-type '(windows-nt darwin))
+  (let ((l (assoc "xdvi" TeX-view-program-list-builtin)))
+    (when l
+      (setcar (cadr l) "%(xdvi) -unique")
+      (setcdr (cdr l) '("%(xdvi)"))))
+  (setq TeX-view-program-list-builtin
+	(append TeX-view-program-list-builtin
+		'(("MuPDF" "mupdf %o" "mupdf")))))
 
 ;; $B$3$l$O(B tex.el $B$K<h$jF~$l$F$b$i$&$N$OFq$7$$$+!)(B
 ;; tex-jp.el $B$,FI$_9~$^$l$k$@$1$G!"(Bdvi viewer $B$N%G%U%)%k%H$,(B dviout $B$K(B
 ;; $B$J$C$F$7$^$&$N$ODq93$,Bg$-$$$+$b!#(B
 (unless (get 'TeX-view-program-selection 'saved-value)
-  (setq TeX-view-program-selection
-       (append
-        (cond
-         ((eq system-type 'windows-nt)
-          '((output-dvi "Dviout")
-            (output-pdf "TeXworks")))
-         ((eq system-type 'darwin)
-          '((output-pdf "Preview")))
-         (t
-          nil))
-        TeX-view-program-selection)))
+  (if (eq system-type 'windows-nt)
+      (setq TeX-view-program-selection
+	    (append
+	     '((output-dvi "dviout"))
+	     TeX-view-program-selection))))
 
 (mapc (lambda (dir) (add-to-list 'TeX-macro-global dir t))
       (or (TeX-tree-expand
@@ -258,61 +224,24 @@ For detail, see `TeX-command-list', to which this list is appended."
 	   "jlatex" '("/jtex/" "/jbibtex/bst/"))
 	  '("/usr/share/texmf/jtex/" "/usr/share/texmf/jbibtex/bst/")))
 
-;; $B=gD4$K9T$1$PITMW$K$J$k!#(B
-(setq LaTeX-command-style
-      (append '(("\\`u[jt]\\(article\\|report\\|book\\)\\'\\|\\`uplatex\\'"
-                "%(PDF)uplatex %(kanjiopt)%S%(PDFout)")
-               ("\\`[jt]s?\\(article\\|report\\|book\\)\\'"
-                "%(PDF)platex %(kanjiopt)%S%(PDFout)")
-               ("\\`j-\\(article\\|report\\|book\\)\\'"
-                "%(PDF)jlatex %(kanjiopt)%S%(PDFout)"))
-	      LaTeX-command-style))
-
 (defcustom japanese-TeX-error-messages t
   "*If non-nil, explain TeX error messages in Japanese."
   :group 'AUCTeX-jp
   :type 'boolean)
 
-(when (featurep 'mule)
-
-;; FIX-ME (2007-02-09) The default coding system in recent Unix (like Fedora and
-;; Ubuntu) is utf-8.  But Japanese TeX system does not support utf-8 yet
-;; (platex-utf is under development, may be alpha phase).  So,
-;; process-coding-system for Japanese TeX is not defined from
-;; default-coding-system.  When platex-utf is out, we should look this setting,
-;; again.
-
-(defcustom TeX-japanese-process-input-coding-system
-  (cond ((memq system-type '(windows-nt ms-dos cygwin)) 'shift_jis-dos)
-	((memq system-type '(mac darwin)) 'shift_jis-mac)
-	(t 'euc-jp-unix))
-  "TeX-process' coding system with standard input."
+(defcustom TeX-japanese-process-input-coding-system nil
+  "If non-nil, used for encoding input to Japanese TeX process.
+When nil, AUCTeX tries to choose suitable coding system.
+See also a user custom option `TeX-japanese-process-output-coding-system'."
   :group 'AUCTeX-jp
-  :type 'coding-system)
+  :type '(choice (const :tag "Default" nil) coding-system))
 
-(defcustom TeX-japanese-process-output-coding-system
-  (cond ((memq system-type '(windows-nt ms-dos cygwin)) 'shift_jis-dos)
-	((memq system-type '(mac darwin)) 'shift_jis-mac)
-	(t 'euc-jp-unix))
-  "TeX-process' coding system with standard output."
+(defcustom TeX-japanese-process-output-coding-system nil
+  "If non-nil, used for decoding output from Japanese TeX process.
+When nil, AUCTeX tries to choose suitable coding system.
+See also a user custom option `TeX-japanese-process-input-coding-system'."
   :group 'AUCTeX-jp
-  :type 'coding-system)
-
-)
-
-;; $B=gD4$K9T$1$PITMW$K$J$k!#(B
-(defcustom japanese-TeX-command-default "pTeX"
-  "*The default command for `TeX-command' in the japanese-TeX mode."
-  :group 'AUCTeX-jp
-  :type 'string)
-  (make-variable-buffer-local 'japanese-TeX-command-default)
-
-;; $B=gD4$K9T$1$PITMW$K$J$k!#(B
-(defcustom japanese-LaTeX-command-default "LaTeX"
-  "*The default command for `TeX-command' in the japanese-LaTeX mode."
-  :group 'AUCTeX-jp
-  :type 'string)
-  (make-variable-buffer-local 'japanese-LaTeX-command-default)
+  :type '(choice (const :tag "Default" nil) coding-system))
 
 (defcustom japanese-LaTeX-default-style "jarticle"
   "*Default when creating new Japanese documents."
@@ -331,31 +260,113 @@ For detail, see `TeX-command-list', to which this list is appended."
     ("treport")
     ("tbook")
     ("jsarticle")
-    ("jsbook"))
-  "*List of Japanese document styles."
+    ("jsbook")
+    ;; for upLaTeX
+    ("ujarticle") ("ujreport") ("ujbook")
+    ("utarticle") ("utreport") ("utbook"))
+  "*List of Japanese document classes."
   :group 'AUCTeX-jp
   :type '(repeat (group (string :format "%v"))))
 
 (setq LaTeX-style-list
       (append japanese-LaTeX-style-list LaTeX-style-list))
 
-;;; Coding system
+;; text$B!A7O$NL@D+BN!&%4%7%C%/BN;XDj%3%^%s%I$O(B jLaTeX $B$K$O$J$$$h$&$G!"(B
+;; (u)pLaTeX $B$G$7$+;H$($J$$$,!"LdBj$K$J$k$3$H$O$J$$$@$m$&!#(B
+(setq LaTeX-font-list
+      (append '((?m "\\textmc{" "}" "\\mathmc{" "}")
+                (?g "\\textgt{" "}" "\\mathgt{" "}"))
+	      LaTeX-font-list))
 
-(when (featurep 'mule)
+;;; Coding system
 
 (defun japanese-TeX-set-process-coding-system (process)
   "Set proper coding system for japanese TeX PROCESS."
-  (if (with-current-buffer TeX-command-buffer japanese-TeX-mode)
-      (set-process-coding-system process
-				 TeX-japanese-process-output-coding-system
-				 TeX-japanese-process-input-coding-system)))
-(setq TeX-after-start-process-function
-      'japanese-TeX-set-process-coding-system)
+  (with-current-buffer TeX-command-buffer
+    (when japanese-TeX-mode
+      ;; TeX-engine $B$,(B ptex, jtex, uptex $B$N$$$:$l$+$G$"$k>l9g$N$_9M$((B
+      ;; $B$k!#(Bluatex-ja $B$J$I$N>l9g$O$=$b$=$b$?$@$N(B latex-mode $B$G$h$/!"(B
+      ;; $B$o$6$o$6(B japanese-latex-mode $B$K$9$kI,MW$,$J$$!#(B
 
-(defcustom japanese-TeX-use-kanji-opt-flag t
-  "Add kanji option to Japanese pTeX family if non-nil."
-  :group 'AUCTeX-jp
-  :type 'boolean)
+      ;; FIXME: $B0J2<$N=hM}$O(B tex engine $B$rBP>]$H$9$k>l9g$7$+9M$($F$$$J$$!#(B
+      ;; bibtex $B$d(B mendex $BEy$NJd=u%D!<%k$N>l9g$O@5$7$/$J$$=hM}$+$b$7$l$J$$!#(B
+      (let*
+	  ;; -kanji $B%*%W%7%g%s$"$j$N;~$NJ8;z%3!<%I!#(B
+	  ((kanji (and japanese-TeX-use-kanji-opt-flag
+		       (let ((str (japanese-TeX-get-encoding-string)))
+			 (cond
+			  ((equal str "euc") 'euc-jp)
+			  ((equal str "jis") 'iso-2022-jp)
+			  ((equal str "sjis") 'shift_jis)
+			  ((equal str "utf8") 'utf-8)))))
+
+	   ;; process $B$+$i$N=PNO$NJ8;z%3!<%I!#(B
+	   (dec (cond
+		 ;; windows $B$H(B mac $B$N>l9g!#(B
+		 ((memq system-type '(windows-nt darwin))
+		  (cond
+		   ;; ptex $B$J$i(B mac $B$O(B utf-8$B!#(B
+		   ;; windows $B$G(B -kanji $B%*%W%7%g%s$"$j$N;~$O$=$NJ8;z%3!<%I!"(B
+		   ;; $B$J$7$N;~$O(B sjis$B!#(B
+		   ;; texlive 2018 $B$+$i$O(B sjis $B$G$O$J$/(B utf-8 $B$K$J$C$?$N$G(B
+		   ;; $B$=$A$i$K9g$o$;$k!#(B
+		   ((eq TeX-engine 'ptex)
+		    (cond ((eq system-type 'darwin)
+			   'utf-8)
+			  ((and japanese-TeX-use-kanji-opt-flag kanji)
+			   kanji)
+			  (t 'utf-8)))
+		   ;; jtex $B$J$i(B sjis $B$K8GDj$9$k!#(B
+		   ((eq TeX-engine 'jtex)
+		    'shift_jis)
+		   ;; uptex $B$J$i(B utf-8 $B$K8GDj$9$k!#(B
+		   (t
+		    'utf-8)))
+		 ;; unix $B$N>l9g!#(B
+		 (t
+		  ;; jtex $B$J$i(B euc $B$K8GDj$9$k!#(B
+		  (cond
+		   ((eq TeX-engine 'jtex)
+		    'euc-jp)
+		   ;; $B$=$l0J30$O!"(Buptex $B$G$b(B locale $B$K=>$&!#(B
+		   ;; $B$?$@$7!"(Blocale $B$,F|K\8l$r%5%]!<%H$7$J$$>l9g$O(B
+		   ;; euc $B$K8GDj$9$k!#(B
+		   (t
+		    (let ((lcs locale-coding-system))
+		      (if (and lcs (japanese-TeX-coding-ejsu lcs))
+			  lcs 'euc-jp)))))))
+
+	   ;; process $B$KM?$($kF~NO$NJ8;z%3!<%I!#(B
+	   (enc (cond
+		 ;; ptex $B$G(B -kanji $B%*%W%7%g%s$"$j$J$i!"$=$NJ8;z%3!<%I!#(B
+		 ;; $B$J$7$J$i(B utf-8 $B$+(B sjis$B!#(B
+		 ;; texlive 2018 $B$G(B w32 $B$G$b(B utf-8 $B$,%G%U%)%k%H$K$J$C(B
+		 ;; $B$?$h$&$J$N$G!"$=$l$K9g$o$;$k!#(B
+		 ((eq TeX-engine 'ptex)
+		  (if (and japanese-TeX-use-kanji-opt-flag kanji)
+		      kanji
+		    'utf-8))
+		 ;; jtex $B$J$i(B euc $B$+(B sjis $B$K8GDj$9$k!#(B
+		 ((eq TeX-engine 'jtex)
+		  (if (memq system-type '(windows-nt darwin))
+		      'shift_jis 'euc-jp))
+		 ;; uptex $B$J$i(B utf-8 $B$K8GDj$9$k!#(B
+		 (t
+		  'utf-8))))
+
+	;; $B2~9T%3!<%I$r;XDj!#(B
+	(setq dec (coding-system-change-eol-conversion
+		   dec
+		   (if (eq system-type 'windows-nt) 'dos 'unix))
+	      enc (coding-system-change-eol-conversion
+		   enc
+		   (if (eq system-type 'windows-nt) 'dos 'unix)))
+
+	;; Customize $BCM$,$"$l$P$=$l$rM%@h!#(B
+	(set-process-coding-system
+	 process
+	 (or TeX-japanese-process-output-coding-system dec)
+	 (or TeX-japanese-process-input-coding-system enc))))))
 
 (defun japanese-TeX-coding-ejsu (coding-system)
   "Convert japanese CODING-SYSTEM to mnemonic string.
@@ -370,8 +381,9 @@ Return nil otherwise."
                 (iso-2022-jp . "jis")
                 (japanese-shift-jis . "sjis")
                 (utf-8 . "utf8")
+		;; TeXLive 2018 $B$+$i(B BOM $B$D$-(B UTF-8 $B$b%5%]!<%H$5$l$?!#(B
+                (utf-8-with-signature . "utf8")
 
-                ;; xemacs $B$@$H0J2<$NL>A0$O0c$&$+$b!D!#(B
                 (euc-jis-2004 . "euc")
                 (iso-2022-jp-2004 . "jis")
                 (japanese-shift-jis-2004 . "sjis")
@@ -400,15 +412,8 @@ For inappropriate encoding, nil instead."
       (japanese-TeX-coding-ejsu
        (default-value 'buffer-file-coding-system))))
 
-)
-
 ;;; Japanese TeX modes
 
-(defvar japanese-TeX-mode nil
-  "Non-nil means the current buffer handles Japanese TeX/LaTeX.")
-(make-variable-buffer-local 'japanese-TeX-mode)
-(put 'japanese-TeX-mode 'permanent-local t)
-
 ;;;###autoload
 (defun japanese-plain-tex-mode ()
   "Major mode in AUCTeX for editing Japanese plain TeX files.
@@ -420,10 +425,16 @@ Set `japanese-TeX-mode' to t, and enter `TeX-plain-tex-mode'."
 (defun japanese-plain-tex-mode-initialization ()
   "Japanese plain-TeX specific initializations."
   (when japanese-TeX-mode
-;    (setq TeX-command-default japanese-TeX-command-default)
-    (TeX-engine-set japanese-TeX-engine-default)))
+    (TeX-engine-set japanese-TeX-engine-default)
 
-(add-hook 'plain-TeX-mode-hook 'japanese-plain-tex-mode-initialization)
+    ;; For the intent of the following lines, see the comments below
+    ;; in `japanese-latex-mode-initialization'.
+    (when enable-local-variables
+      (setq major-mode 'japanese-plain-tex-mode)
+      (add-hook 'hack-local-variables-hook #'japanese-TeX-reset-mode-name
+		nil t))))
+
+(add-hook 'plain-TeX-mode-hook #'japanese-plain-tex-mode-initialization)
 
 ;;;###autoload
 (defun japanese-latex-mode ()
@@ -436,39 +447,91 @@ Set `japanese-TeX-mode' to t, and enter `TeX-latex-mode'."
 (defun japanese-latex-mode-initialization ()
   "Japanese LaTeX specific initializations."
   (when japanese-TeX-mode
-;    (setq TeX-command-default japanese-LaTeX-command-default)
-    (TeX-engine-set
-     ;; class file $BL>$KMj$k$N$O@5$7$$$N$+!)(B
-     ;; jLaTeX $B$K$b(B jarticle $B$O0l1~$"$k$7!"(BpLaTeX $B$G$b<+J,$G(B j-article $B$r(B
-     ;; $B%$%s%9%H!<%k$7$F;H$C$F$$$1$J$$K!$O$J$$!#(B
-     (cond
-      ((TeX-match-style "\\`u[jt]\\(article\\|report\\|book\\)\\'\\|\\`uplatex\\'")
-       'uptex)
-      ((TeX-match-style "\\`[jt]s?\\(article\\|report\\|book\\)\\'")
-       'ptex)
-      ((TeX-match-style "\\`j-\\(article\\|report\\|book\\)\\'")
-       'jtex)
-      (t japanese-TeX-engine-default)))
+    ;; `TeX-match-style' $B$r;H$&$N$O(B `TeX-update-style' $B$N8e$KCY$i$;$k!#(B
+    ;; $B$3$NCJ3,$G;H$&$H!"$=$NCf$G8F$V(B `TeX-style-list' $B$NCf$G(B
+    ;; `TeX-update-style' $B$,8F$P$l$F$7$^$$!"(Blocal variable $BEy$N=`Hw$,(B
+    ;; $B@0$C$F$J$$CJ3,$G(B style hook $B$,<B9T$5$l$FITE,$J7k2L$K$J$k$3$H$,(B
+    ;; $B$"$k!#$^$?!"(B`TeX-update-style' $B$O8e$+$i(B `find-file-hook' $BCf$G$b(B
+    ;; $B$&0lEY8F$P$l$k$N$G!"(B`TeX-parse-self' $B$,(B t $B$@$H(B parse $B=hM}$bL5BL(B
+    ;; $B$K(B 2 $B2s9T$o$l$F$7$^$&!#(B
+    (add-hook 'TeX-update-style-hook
+	      #'japanese-LaTeX-guess-engine nil t)
     (setq LaTeX-default-style japanese-LaTeX-default-style)
-;    (setq TeX-command-BibTeX
-;        (if (and (eq TeX-engine 'ptex) (executable-find "pbibtex"))
-;            "pBibTeX" "jBibTeX"))
-))
 
-(add-hook 'LaTeX-mode-hook 'japanese-latex-mode-initialization)
+    (when (and (fboundp 'font-latex-add-keywords)
+	       (eq TeX-install-font-lock 'font-latex-setup))
+      ;; jLaTeX $B$K$O$J$$%3%^%s%I$@$,!"$=$l$O$b$&5$$K$7$J$/$F$$$$$@$m$&!#(B
+      (font-latex-add-keywords '(("textgt" "{") ("mathgt" "{"))
+			       'bold-command)
+      (font-latex-add-keywords '("gtfamily")
+			       'bold-declaration))
 
+    ;; The value of `major-mode' should be `latex-mode', not
+    ;; `japanese-latex-mode', because the name `latex-mode' is hard
+    ;; coded in several places of AUCTeX like "(eq major-mode
+    ;; 'latex-mode)", "(memq major-mode '(doctex-mode latex-mode)" and
+    ;; so on.  By such piece of codes, `japanese-latex-mode' should
+    ;; simply be regarded as `latex-mode'.  So we'd like to leave
+    ;; `major-mode' as `latex-mode' here, but doing so confuses
+    ;; `hack-local-variables' in two ways.
+    ;; (1) It is tricked into considering that the major mode is not
+    ;;     yet initialized and calls `japanese-latex-mode' again.
+    ;; (2) It does not read the directory local variables prepared for
+    ;;     `japanese-latex-mode'.
+    ;; Thus we temporarily set `major-mode' to `japanese-latex-mode'
+    ;; here and plan to reset it to `latex-mode' after
+    ;; `hack-local-variables' is done.
+    (when enable-local-variables
+      (setq major-mode 'japanese-latex-mode)
+      (add-hook 'hack-local-variables-hook #'japanese-TeX-reset-mode-name
+		nil t))))
+
+(add-hook 'LaTeX-mode-hook #'japanese-latex-mode-initialization)
+
+;; This function is useful only within `hack-local-variables-hook'.
+(defun japanese-TeX-reset-mode-name ()
+  (cond ((eq major-mode 'japanese-latex-mode)
+	 (setq major-mode 'latex-mode))
+	((eq major-mode 'japanese-plain-tex-mode)
+	 (setq major-mode 'plain-tex-mode)))
+  (remove-hook 'hack-local-variables-hook #'japanese-TeX-reset-mode-name t))
+
+;; Make `hack-dir-local-variables' to regard `latex-mode' as parent
+;; of `japanese-latex-mode', and `plain-tex-mode' as parent of
+;; `japanese-plain-tex-mode'.
+(put 'japanese-plain-tex-mode 'derived-mode-parent 'plain-tex-mode)
+(put 'japanese-latex-mode 'derived-mode-parent 'latex-mode)
+
+(defun japanese-LaTeX-guess-engine ()
+  "Guess Japanese TeX engine and set it to `TeX-engine'.
+Document class and its option is considered in the guess.  Do not
+overwrite the value already set locally."
+  ;; `TeX-engine' may be set by the file local variable or by the menu
+  ;; Command->TeXing Options manually.  Don't override the user
+  ;; preference set in such ways.
+  (unless (local-variable-p 'TeX-engine (current-buffer))
+    (TeX-engine-set
+     (cond
+      ((TeX-match-style "\\`u[jt]\\(?:article\\|report\\|book\\)\\'")
+       'uptex)
+      ((TeX-match-style "\\`[jt]s?\\(?:article\\|report\\|book\\)\\'")
+       (if (LaTeX-match-class-option "\\`uplatex\\'")
+	   'uptex 'ptex))
+      ((TeX-match-style "\\`j-\\(?:article\\|report\\|book\\)\\'")
+       'jtex)
+      (t japanese-TeX-engine-default)))))
 
 ;;; Support for various self-insert-command
 
-(fset 'japanese-TeX-self-insert-command
-      (cond ((fboundp 'can-n-egg-self-insert-command)
-	     'can-n-egg-self-insert-command)
-	    ((fboundp 'egg-self-insert-command)
-	     'egg-self-insert-command)
-	    ((fboundp 'canna-self-insert-command)
-	     'canna-self-insert-command)
-	    (t
-	     'self-insert-command)))
+(defalias 'japanese-TeX-self-insert-command
+  (cond ((fboundp 'can-n-egg-self-insert-command)
+	 #'can-n-egg-self-insert-command)
+	((fboundp 'egg-self-insert-command)
+	 #'egg-self-insert-command)
+	((fboundp 'canna-self-insert-command)
+	 #'canna-self-insert-command)
+	(t
+	 #'self-insert-command)))
 
 (defun TeX-insert-punctuation ()
   "Insert point or comma, cleaning up preceding space."
@@ -476,13 +539,18 @@ Set `japanese-TeX-mode' to t, and enter `TeX-latex-mode'."
   (expand-abbrev)
   (if (TeX-looking-at-backward "\\\\/\\(}+\\)" 50)
       (replace-match "\\1" t))
-  (call-interactively 'japanese-TeX-self-insert-command))
+  (call-interactively #'japanese-TeX-self-insert-command))
 
 ;;; Error Messages
 
 (if japanese-TeX-error-messages
 (setq TeX-error-description-list
-  '(("Bad \\\\line or \\\\vector argument.*" .
+  '(("\\(?:Package Preview Error\\|Preview\\):.*" .
+"`preview'$B$X(B`auctex'$B%*%W%7%g%s$rD>@\M?$($k$N$OHr$1$F$/$@$5$$!%(B
+$B%W%l%S%e!<$N<B9T;~0J30$G$3$N%(%i!<$,=P$?>l9g!$M>$j$K$3$_$$$C$?$3$H$r(B
+$B$7$9$.$+!$$G$J$1$l$P(BAUCTeX$B$,$R$I$$<:GT$r$7$F$$$^$9!%(B")
+
+    ("Bad \\\\line or \\\\vector argument.*" .
 "$B@~$N79$-$r;XDj$9$k!$(B\\line$B$^$?$O(B\\vector$B$N:G=i$N0z?t$,IT@5$G$9!%(B")
 
     ("Bad math environment delimiter.*" .
